本文档描述了百度阿波罗计划中端到端驾驶平台的数据格式及相关评测标准。

# 数据集简介

数据是通过百度自己的地图采集车采集而来。 目前，该数据覆盖了中国整个公路网，总长达百万公里。本数据集提供车前图像和车辆运动状态两种类型数据。地图采集车辆捕获360度视图图像，但是由于文件大小限制，我们只提供320 * 320分辨率的车前图像jpg格式数据。 车辆运动状态数据包括当前速度和轨迹曲率。
# 采集情况
数据采集设备如图所示，CCD 相机的光圈应为 2.8，焦距应为∞。摄像头位于车上方顶盖中。通过调整行李架卡扣前后位置,保证行李架本身与车辆中心线垂直，且两条行李架在同一水平面上。将设备支架移动至行李支架中间，保证 CCD 拍摄图像的中间与车辆中心线重合。
 <img src="./png_resource/1.png" width = "300" height = "320" align=center /> 
 <img src="./png_resource/2.png" width = "300" height = "320" align=center />
# 数据用途
数据主要来源于传感器的原始数据，包括图像、激光雷达、雷达等。end-to-end输入以图像为主。 输出是车辆的控制决策指令，如方向盘角度、加速、刹车。 连接输入输出的是深度神经网络，即通过神经网络直接生成车辆控制指令对车辆进行横向控制和纵向控制，中间没有人工参与的逻辑程序。横向控制，主要是指通过方向盘控制车身横向移动，即方向盘角度。纵向控制，是指通过油门和刹车控制车身纵向的移动，即加速、刹车等。横向模型的输出没有采用方向盘角度，而是使用要行驶的曲率（即拐弯半径的倒数）。原因如下：
（1）曲率更普适，不受车辆自身参数如steering ratio、wheel base等影响。
（2）曲率与方向盘角度之间的函数关系简单，低速时通过Ackermann模型就可以反演，高速时通过一个简单的网络也可以拟合。故得到的横向控制模型就是：通过前向的影像，给出车辆要行驶的曲率。纵向模型的输出是加速度。




# 数据分类
数据分为两部分，训练集和测试集。使用训练集调试算法，使用测试集检验模型。
## 训练集
训练集包含图像image和姿态attr两部分数据。image为输入数据，attr为输出数据。其中image文件和attr文件通过文件名一一对应，读取image和attr需要hdf5库支持。
### 训练集格式
* 训练数据按照如下目录结构组织：

	```
trainsets/				    // 训练数据的顶层文件夹
├── image					 // 包含了训练数据中的图像文件
│   ├── 119.h5
│   ├── ..
│   └── 123.h5
└── attr					// 包含了与图像文件对应的姿态文件
    ├── 119.h5
    ├── ..
    └── 123.h5   
	```


* 图像文件：h5文件。key-value格式，以时间戳UTC time为索引检索一幅图像，key:UTC time， value:320 * 320 * 3的图像编码后jpg格式数据。解码示例：

	```
	with h5py.File(img_file, 'r') as f:
		for t in f:
			img = cv2.imdecode(f[t][:], 1)
	
	```

* 姿态文件：h5文件。一个时刻的姿态数据以二维数组整体存在hdf5中，第一维为’attrs’,第二维为姿态数据：［t, VEast, VNorth, curv1, curv2, curv3, curv4, curv5, curv6, x, y, heading, tag］。对于图像文件中的一个UTC time的图像，一定存在一行与之对应的姿态数据，每一行有13个数据，类型为64位浮点数。字段说明如下：

	列：变量 | 单位 | 说明
	------------ | ------------- | ------------
	01 : t | 十进制（无符号） | 当前UTC time 时间戳
	02 : VEast|	米／秒 | 当前车辆朝东的速度
	03 : VNorth|	米／秒 | 当前车辆朝北的速度
	04 : curv1 | 十进制（有符号) | [t,t+0.125)间曲率1，左拐为正
	05 : curv2 | 十进制（有符号） | [t,t+0.125)间曲率2，左拐为正
	06 : curv3 | 十进制（有符号） | [t,t+0.125)间曲率3，左拐为正
	07 : curv4 | 十进制（有符号） | [t,t+0.125)间曲率4，左拐为正
	08 : curv5 | 十进制（有符号） | [t,t+0.125)间曲率5，左拐为正
	09 : curv6 | 十进制（有符号） | [t,t+0.125)间曲率6，左拐为正
	10 : x | 十进制（无符号） | 与正东向x轴的相对位移
	11 : y | 十进制（无符号） | 与正北向y轴的相对位移
	12 : heading | 度，十进制浮点数（有符号） | 与北向的顺时针夹角
	13 : tag | 十进制（无符号） | 保留的标记位，暂未使用

# 评测数据格式
* 测试集只包含图像数据，格式同训练集。测试数据按照如下目录结构组织：

	```
testsets/				    // 测试数据的顶层文件夹
├── image					 // 包含了测试数据中的图像文件
│   ├── testfile_part01.h5
│   ├── ..
│   └── testfile_part04.h5
└── attr					// 包含了与图像文件对应的姿态文件
    ├── testfile_part01.h5
    ├── ..
    └── testfile_part04.h5 
	```
* 使用测试集预测曲率和加速度。预测结果写入.h5文件，二维数组整体存在hdf5中，浮点数存储。

   列：变量 | 单位 | 说明
	------------ | ------------- | ------------
	01 : t | 十进制（无符号） | 当前UTC time 时间戳
	02 : curv|	十进制（有符号）|	[t,t+0.125）间曲率，左拐为正
	03: acc	|十进制（有符号）|[t,t+0.125）间加速度，加速为正

# 评测指标说明
横纵向模型预测效果使用均方差(MSE)指标来衡量。均方差是指预测值与真实值差值的平方的均值，均方差值越小，效果越好。该指标用来衡量预测的精确性，定义公式如下：
<script type="text/javascript" src="http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=default"></script>

$$ MSE = \frac1{n}\sum_{i=1}^n{(truth- predict )^2} $$
