/******************************************************************************
 * Copyright 2017 The Apollo Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *****************************************************************************/

/**
 * @file
 * @brief: Definition of PiecewiseLinearKernel class
 **/

#ifndef MODULES_PLANNING_MATH_SMOOTHING_SPLINE_PIECEWISE_LINEAR_KERNEL_H_
#define MODULES_PLANNING_MATH_SMOOTHING_SPLINE_PIECEWISE_LINEAR_KERNEL_H_

#include <vector>

#include "Eigen/Core"

namespace apollo {
namespace planning {

class PiecewiseLinearKernel {
 public:
  PiecewiseLinearKernel(const uint32_t dimension, const double unit_segment);

  void AddRegularization(const double regularized_param);

  const Eigen::MatrixXd& kernel_matrix() const;
  const Eigen::MatrixXd& offset_matrix() const;

  void AddDerivativeKernelMatrix(const double weight);
  void AddSecondOrderDerivativeMatrix(const double weight);
  void AddThirdOrderDerivativeMatrix(const double weight);

  // reference line kernel
  bool AddReferenceLineKernelMatrix(const std::vector<uint32_t>& index_list,
                                    const std::vector<double>& pos_list,
                                    const double weight);

 private:
  const uint32_t dimension_;
  const double unit_segment_;
  Eigen::MatrixXd kernel_matrix_;
  Eigen::MatrixXd offset_matrix_;
};

}  // namespace planning
}  // namespace apollo

#endif  // MODULES_PLANNING_MATH_SMOOTHING_SPLINE_PIECEWISE_LINEAR_KERNEL_H_
